## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
##
## Implementaion of miscellaneous game functions

import CvUtil
from CvPythonExtensions import *
import CvEventInterface
import CustomFunctions
import ScenarioFunctions
import PyHelpers

# globals
cf = CustomFunctions.CustomFunctions()
gc = CyGlobalContext()
sf = ScenarioFunctions.ScenarioFunctions()
PyPlayer = PyHelpers.PyPlayer

class CvGameUtils:
    "Miscellaneous game functions"
    def __init__(self): 
        pass
    
    def isVictoryTest(self):
        if ( gc.getGame().getElapsedGameTurns() > 10 ):
            return True
        else:
            return False

    def isVictory(self, argsList):
        eVictory = argsList[0]
        return True

    def isPlayerResearch(self, argsList):
        ePlayer = argsList[0]
        return True

    def getExtraCost(self, argsList):
        ePlayer = argsList[0]
        return 0

    def createBarbarianCities(self):
        return False
        
    def createBarbarianUnits(self):
        return False
        
    def skipResearchPopup(self,argsList):
        ePlayer = argsList[0]
        return False
        
    def showTechChooserButton(self,argsList):
        ePlayer = argsList[0]
        return True

    def getFirstRecommendedTech(self,argsList):
        ePlayer = argsList[0]
        return TechTypes.NO_TECH

    def getSecondRecommendedTech(self,argsList):
        ePlayer = argsList[0]
        eFirstTech = argsList[1]
        return TechTypes.NO_TECH
    
    def canRazeCity(self,argsList):
        iRazingPlayer, pCity = argsList
        return True
    
    def canDeclareWar(self,argsList):
        iAttackingTeam, iDefendingTeam = argsList
        return True
    
    def skipProductionPopup(self,argsList):
        pCity = argsList[0]
        return False
        
    def showExamineCityButton(self,argsList):
        pCity = argsList[0]
        return True

    def getRecommendedUnit(self,argsList):
        pCity = argsList[0]
        return UnitTypes.NO_UNIT

    def getRecommendedBuilding(self,argsList):
        pCity = argsList[0]
        return BuildingTypes.NO_BUILDING

    def updateColoredPlots(self):
        return False

    def isActionRecommended(self,argsList):
        pUnit = argsList[0]
        iAction = argsList[1]
        return False

    def unitCannotMoveInto(self,argsList):
        ePlayer = argsList[0]       
        iUnitId = argsList[1]
        iPlotX = argsList[2]
        iPlotY = argsList[3]
        return False

    def cannotHandleAction(self,argsList):
        pPlot = argsList[0]
        iAction = argsList[1]
        bTestVisible = argsList[2]
        return False

    def canBuild(self,argsList):
        iX, iY, iBuild, iPlayer = argsList
        pPlayer = gc.getPlayer(iPlayer)
        eTeam = gc.getTeam(pPlayer.getTeam())
        
        if pPlayer.isHuman() == False:  
            if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CONSTRUCTION')) == True :
                if pPlayer.getCivics(gc.getInfoTypeForString('CIVICOPTION_GOVERNMENT')) == gc.getInfoTypeForString('CIVIC_ARISTOCRACY'):
                    if pPlayer.getCivics(gc.getInfoTypeForString('CIVICOPTION_ECONOMY')) == gc.getInfoTypeForString('CIVIC_AGRARIANISM'):
                        if iBuild == gc.getInfoTypeForString('BUILD_COTTAGE'):
                            return 0
                        if iBuild == gc.getInfoTypeForString('BUILD_LUMBERMILL'):
                            return 0

            if pPlayer.getCivics(gc.getInfoTypeForString('CIVICOPTION_LABOR')) == gc.getInfoTypeForString('CIVIC_ARETE'):
                if iBuild == gc.getInfoTypeForString('BUILD_WINDMILL'):
                    pPlotCity = plotXY(iX, iY, 0, 0).getWorkingCity()
                    if pPlotCity.foodDifference(False) > 0:
                        return 0
                
            if eTeam.isHasTech(gc.getInfoTypeForString('TECH_GUILDS')) == False :
                if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_FALLOW')) == False:  
                    if iBuild == gc.getInfoTypeForString('BUILD_WORKSHOP'):
                        return 0

            if gc.getBuildInfo(iBuild).getType()[0:11] == 'BUILD_MANA_':
                if pPlayer.isHasTech(gc.getInfoTypeForString('TECH_ELEMENTALISM')):
                    # These civs always need a fire node
                    if pPlayer.getCivilizationType() in (gc.getInfoTypeForString('CIVILIZATION_LUCHUIRP'), gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR'), gc.getInfoTypeForString('CIVILIZATION_SVARTALFAR')):
                        if not pPlayer.hasBonus(gc.getInfoTypeForString('BONUS_MANA_FIRE')):
                            if iBuild != gc.getInfoTypeForString('BUILD_MANA_FIRE'):
                                return 0
                
        return -1   # Returning -1 means ignore; 0 means Build cannot be performed; 1 or greater means it can

    def cannotFoundCity(self,argsList):
        iPlayer, iPlotX, iPlotY = argsList
        pPlot = CyMap().plot(iPlotX,iPlotY)
        return False

    def cannotSelectionListMove(self,argsList):
        pPlot = argsList[0]
        bAlt = argsList[1]
        bShift = argsList[2]
        bCtrl = argsList[3]
        return False

    def cannotSelectionListGameNetMessage(self,argsList):
        eMessage = argsList[0]
        iData2 = argsList[1]
        iData3 = argsList[2]
        iData4 = argsList[3]
        iFlags = argsList[4]
        bAlt = argsList[5]
        bShift = argsList[6]
        return False

    def cannotDoControl(self,argsList):
        eControl = argsList[0]
        return False

    def canResearch(self,argsList):
        ePlayer = argsList[0]
        eTech = argsList[1]
        bTrade = argsList[2]
        return False

    def cannotResearch(self,argsList):
        ePlayer = argsList[0]
        eTech = argsList[1]
        bTrade = argsList[2]
        pPlayer = gc.getPlayer(ePlayer)

        if eTech == gc.getInfoTypeForString('TECH_ORDERS_FROM_HEAVEN'):
            if gc.getGame().isOption(GameOptionTypes.GAMEOPTION_NO_RELIGION_1):
                return True
            if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')):
                return True
            if pPlayer.isHuman() == False:
                if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_EVIL'):
                    return True
        if eTech == gc.getInfoTypeForString('TECH_WAY_OF_THE_EARTHMOTHER'):
            if gc.getGame().isOption(GameOptionTypes.GAMEOPTION_NO_RELIGION_3):
                return True
            if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')):
                return True
            if pPlayer.isHuman() == False:
                if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_EVIL'):
                    return True
        if eTech == gc.getInfoTypeForString('TECH_WAY_OF_THE_FORESTS'):
            if gc.getGame().isOption(GameOptionTypes.GAMEOPTION_NO_RELIGION_0):
                return True
            if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')):
                return True
        if eTech == gc.getInfoTypeForString('TECH_MESSAGE_FROM_THE_DEEP'):
            if gc.getGame().isOption(GameOptionTypes.GAMEOPTION_NO_RELIGION_2):
                return True
            if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')):
                return True
            if pPlayer.isHuman() == False:
                if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_GOOD'):
                    return True
        if eTech == gc.getInfoTypeForString('TECH_CORRUPTION_OF_SPIRIT'):
            if gc.getGame().isOption(GameOptionTypes.GAMEOPTION_NO_RELIGION_4):
                return True
            if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_AGNOSTIC')):
                return True
            if pPlayer.isHuman() == False:
                if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_GOOD'):
                    return True
        if eTech == gc.getInfoTypeForString('TECH_WAY_OF_THE_WISE'):
            if pPlayer.isHuman() == False:
                if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_EVIL'):
                    return True
        if eTech == gc.getInfoTypeForString('TECH_WAY_OF_THE_WICKED'):
            if pPlayer.isHuman() == False:
                if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_GOOD'):
                    return True
        if eTech == gc.getInfoTypeForString('TECH_HONOR'):
            if pPlayer.isHuman() == False:
                if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_EVIL'):
                    return True
        if eTech == gc.getInfoTypeForString('TECH_DECEPTION'):
            if pPlayer.isHuman() == False:
                if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_GOOD'):
                    return True

        if eTech == gc.getInfoTypeForString('TECH_SEAFARING'):
            if pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_LANUN'):
                return True

        if CyGame().getWBMapScript():
            bBlock = sf.cannotResearch(ePlayer, eTech, bTrade)
            if bBlock:
                return True

        return False

    def canDoCivic(self,argsList):
        ePlayer = argsList[0]
        eCivic = argsList[1]
        return False

    def cannotDoCivic(self,argsList):
        ePlayer = argsList[0]
        eCivic = argsList[1]
        pPlayer = gc.getPlayer(ePlayer)
        eTeam = gc.getTeam(pPlayer.getTeam())
        
        if pPlayer.isHuman() == False :
            if eCivic == gc.getInfoTypeForString('CIVIC_CONQUEST'):
                if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_FALLOW')) == False:  
                    return True

            if eCivic == gc.getInfoTypeForString('CIVIC_MERCANTILISM'):
                return True 

            if eCivic == gc.getInfoTypeForString('CIVIC_DESPOTISM'):
                if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CARTOGRAPHY')):
                    return True     

            if eCivic == gc.getInfoTypeForString('CIVIC_GOD_KING') :
                if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CARTOGRAPHY')):
                    if not pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_KURIOTATES') :
                        if pPlayer.getNumCities() >= 7:
                            return True 

            if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CODE_OF_LAWS')):
                if gc.getCivicInfo(eCivic).getCivicOptionType() == gc.getInfoTypeForString('CIVICOPTION_GOVERNMENT'):
                    bAristocracyValid = False

                    if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_FINANCIAL')) == True:
                        # Only use Aristocracy if Financial is a permanent trait
                        if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_ADAPTIVE')) or gc.getLeaderHeadInfo(pPlayer.getPersonalityType()).getPermanentTrait() == gc.getInfoTypeForString('TRAIT_FINANCIAL'):
                            if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_INSANE')):
                                bAristocracyValid = True     

                    if not bAristocracyValid and eCivic == gc.getInfoTypeForString('CIVIC_ARISTOCRACY'):
                        return True
                    elif bAristocracyValid and eCivic != gc.getInfoTypeForString('CIVIC_ARISTOCRACY'):
                        return True     

            if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_KHAZAD'):
                # Special rules for Khazad - assume aristocracy vs. city states choice is already taken care of above
                if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CARTOGRAPHY')):
                    if gc.getCivicInfo(eCivic).getCivicOptionType() == gc.getInfoTypeForString('CIVICOPTION_GOVERNMENT'):
                        if eCivic not in (gc.getInfoTypeForString('CIVIC_ARISTOCRACY'), gc.getInfoTypeForString('CIVIC_CITY_STATES')):
                            return True

            if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_FALLOW')):
                if eCivic == gc.getInfoTypeForString('CIVIC_AGRARIANISM'):
                    return True

        return False
        
    def canTrain(self,argsList):
        pCity = argsList[0]
        eUnit = argsList[1]
        bContinue = argsList[2]
        bTestVisible = argsList[3]
        bIgnoreCost = argsList[4]
        bIgnoreUpgrades = argsList[5]
        return False

    def cannotTrain(self,argsList):
        pCity = argsList[0]
        eUnit = argsList[1]
        bContinue = argsList[2]
        bTestVisible = argsList[3]
        bIgnoreCost = argsList[4]
        bIgnoreUpgrades = argsList[5]
        ePlayer = pCity.getOwner()
        pPlayer = gc.getPlayer(ePlayer)
        eUnitClass = gc.getUnitInfo(eUnit).getUnitClassType()
        eTeam = gc.getTeam(pPlayer.getTeam())
        pLeader = gc.getLeaderHeadInfo(pPlayer.getPersonalityType())

        if pPlayer.isHuman() == False and bContinue == False:
            infoCiv = gc.getCivilizationInfo(pPlayer.getCivilizationType())
            if eUnitClass == gc.getInfoTypeForString('UNITCLASS_WARRIOR'):
                pPlot = pCity.plot()
                if pPlot.getNumUnits() >= 3:
                    iTrainingYard = infoCiv.getCivilizationBuildings(gc.getInfoTypeForString('BUILDINGCLASS_TRAINING_YARD'))
                    if iTrainingYard != -1:
                        if pPlayer.canConstruct(iTrainingYard, False, False, False):
                            return True
            elif eUnitClass == gc.getInfoTypeForString('UNITCLASS_SCOUT'):
                if CyGame().getGameTurn() > 100:
                    return True # Scouts useless by around this time
                if pPlayer.getUnitClassCount(gc.getInfoTypeForString('UNITCLASS_SCOUT')) > 2:
                    return True
                iHuntingLodge = infoCiv.getCivilizationBuildings(gc.getInfoTypeForString('BUILDINGCLASS_HUNTING_LODGE'))
                if iHuntingLodge != -1:
                    if pPlayer.canConstruct(iHuntingLodge, False, False, False):
                        return True
            elif eUnitClass == gc.getInfoTypeForString('UNITCLASS_SETTLER'):
                if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_KHAZAD'):
                    if pPlayer.getUnitClassCountPlusMaking(gc.getInfoTypeForString('UNITCLASS_SETTLER')) > 1:
                        if not pPlayer.isHasTech(gc.getInfoTypeForString('TECH_WAY_OF_THE_EARTHMOTHER')) and not pPlayer.isCivic(gc.getInfoTypeForString('CIVIC_ARISTOCRACY')) and not pPlayer.isCivic(gc.getInfoTypeForString('CIVIC_CITY_STATES')):
                            return True
                        if not pPlayer.isHasTech(gc.getInfoTypeForString('TECH_FESTIVALS')):
                            return True
                    if pPlayer.getGold() < self.AI_goldTarget([ePlayer]):
                        return True
                if pCity.isCapital() and pPlayer.getNumCities() < 2:
                    if pCity.getPopulation() < 6 and pCity.angryPopulation(1) < 1 and pCity.foodDifference(False) > 1:
                        return True
                if pCity.getPopulation() < 4 and pCity.foodDifference(False) > 1:
                    return True
            elif eUnitClass == gc.getInfoTypeForString('UNITCLASS_WORKER'):
                if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_AGRICULTURE')) and not eTeam.isHasTech(gc.getInfoTypeForString('TECH_CRAFTING')):
                    if pPlayer.getCurrentResearch() not in (gc.getInfoTypeForString('TECH_AGRICULTURE'), gc.getInfoTypeForString('TECH_CRAFTING')):
                        return True
            elif eUnitClass == gc.getInfoTypeForString('UNITCLASS_ADEPT'):
                if pPlayer.getUnitClassCount(gc.getInfoTypeForString('UNITCLASS_ADEPT')) > 3:
                    if not isWorldUnitClass(eUnitClass):
                        if not pPlayer.isHasTech(gc.getInfoTypeForString('TECH_SORCERY')):
                            return True
            elif eUnitClass == gc.getInfoTypeForString('UNITCLASS_DISCIPLE_EMPYREAN'):
                if pPlayer.getUnitClassCount(gc.getInfoTypeForString('UNITCLASS_DISCIPLE_EMPYREAN')) > 3:
                    return True
                if pPlayer.hasHolyCity(gc.getInfoTypeForString('RELIGION_THE_EMPYREAN')) == False :
                    if pLeader.getReligionWeightModifier(gc.getInfoTypeForString('RELIGION_THE_EMPYREAN')) < 25:
                        return True
            elif eUnitClass == gc.getInfoTypeForString('UNITCLASS_DISCIPLE_FELLOWSHIP_OF_LEAVES'):
                if pPlayer.getUnitClassCount(gc.getInfoTypeForString('UNITCLASS_DISCIPLE_FELLOWSHIP_OF_LEAVES')) > 3:
                    return True
                if pPlayer.hasHolyCity(gc.getInfoTypeForString('RELIGION_FELLOWSHIP_OF_LEAVES')) == False :
                    if pLeader.getReligionWeightModifier(gc.getInfoTypeForString('RELIGION_FELLOWSHIP_OF_LEAVES')) < 25:
                        return True
            elif eUnitClass == gc.getInfoTypeForString('UNITCLASS_DISCIPLE_OCTOPUS_OVERLORDS'):
                if pPlayer.getUnitClassCount(gc.getInfoTypeForString('UNITCLASS_DISCIPLE_OCTOPUS_OVERLORDS')) > 3:
                    return True
                if pPlayer.hasHolyCity(gc.getInfoTypeForString('RELIGION_OCTOPUS_OVERLORDS')) == False :
                    if pLeader.getReligionWeightModifier(gc.getInfoTypeForString('RELIGION_OCTOPUS_OVERLORDS')) < 25:
                        return True
            elif eUnitClass == gc.getInfoTypeForString('UNITCLASS_DISCIPLE_RUNES_OF_KILMORPH'):
                if pPlayer.getUnitClassCount(gc.getInfoTypeForString('UNITCLASS_DISCIPLE_RUNES_OF_KILMORPH')) > 3:
                    return True
                if pPlayer.hasHolyCity(gc.getInfoTypeForString('RELIGION_RUNES_OF_KILMORPH')) == False :
                    if pLeader.getReligionWeightModifier(gc.getInfoTypeForString('RELIGION_RUNES_OF_KILMORPH')) < 25:
                        return True
            elif eUnitClass == gc.getInfoTypeForString('UNITCLASS_DISCIPLE_THE_ASHEN_VEIL'):
                if pPlayer.getUnitClassCount(gc.getInfoTypeForString('UNITCLASS_DISCIPLE_THE_ASHEN_VEIL')) > 3:
                    return True
                if pPlayer.hasHolyCity(gc.getInfoTypeForString('RELIGION_THE_ASHEN_VEIL')) == False :
                    if pLeader.getReligionWeightModifier(gc.getInfoTypeForString('RELIGION_THE_ASHEN_VEIL')) < 25:
                        return True
            elif eUnitClass == gc.getInfoTypeForString('UNITCLASS_DISCIPLE_THE_ORDER'):
                if pPlayer.getUnitClassCount(gc.getInfoTypeForString('UNITCLASS_DISCIPLE_THE_ORDER')) > 3:
                    return True
                if pPlayer.hasHolyCity(gc.getInfoTypeForString('RELIGION_THE_ORDER')) == False :
                    if pLeader.getReligionWeightModifier(gc.getInfoTypeForString('RELIGION_THE_ORDER')) < 25:
                        return True
            elif eUnit == gc.getInfoTypeForString('UNIT_DWARVEN_SOLDIER_RUNES'):
                if pPlayer.getUnitClassCount(gc.getInfoTypeForString('UNITCLASS_DWARVEN_SOLDIER_RUNES')) > 2:
                    if eTeam.isHasTech(gc.getInfoTypeForString('TECH_BRONZE_WORKING')): 
                        return True
            elif eUnit == gc.getInfoTypeForString('UNIT_DROWN'):
                if pPlayer.getUnitClassCount(gc.getInfoTypeForString('UNITCLASS_DROWN')) > 2:
                    if eTeam.isHasTech(gc.getInfoTypeForString('TECH_BRONZE_WORKING')): 
                        return True
            elif eUnit == gc.getInfoTypeForString('UNIT_FAWN'):
                if pPlayer.getUnitClassCount(gc.getInfoTypeForString('UNITCLASS_FAWN')) > 2:
                    if eTeam.isHasTech(gc.getInfoTypeForString('TECH_BRONZE_WORKING')): 
                        return True             
            elif eUnit == gc.getInfoTypeForString('UNIT_JAVELIN_THROWER'):
                if 2 * pPlayer.getNumCities() <= pPlayer.getUnitClassCountPlusMaking(gc.getInfoTypeForString('UNIT_JAVELIN_THROWER')) :
                    if eTeam.isHasTech(gc.getInfoTypeForString('TECH_BRONZE_WORKING')): 
                        return True 

            if isWorldUnitClass(eUnitClass):
                if pCity.getPopulation() <=7 and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
                    return True 

        if eUnit == gc.getInfoTypeForString('UNIT_BEAST_OF_AGARES'):
            if pCity.getPopulation() <= 5:
                return True

        if pPlayer.getCivics(gc.getInfoTypeForString('CIVICOPTION_CULTURAL_VALUES')) == gc.getInfoTypeForString('CIVIC_CRUSADE'):
            if eUnit == gc.getInfoTypeForString('UNIT_WORKER'):
                return True
            if eUnit == gc.getInfoTypeForString('UNIT_SETTLER'):
                return True
            if eUnit == gc.getInfoTypeForString('UNIT_WORKBOAT'):
                return True

        if eUnit == gc.getInfoTypeForString('UNIT_ACHERON'):
            if gc.getGame().isOption(GameOptionTypes.GAMEOPTION_NO_ACHERON):
                return True

        if eUnit == gc.getInfoTypeForString('UNIT_DUIN'):
            if gc.getGame().isOption(GameOptionTypes.GAMEOPTION_NO_DUIN):
                return True

        if CyGame().getWBMapScript():
            bBlock = sf.cannotTrain(pCity, eUnit, bContinue, bTestVisible, bIgnoreCost, bIgnoreUpgrades)
            if bBlock:
                return True

        return False

    def canConstruct(self,argsList):
        pCity = argsList[0]
        eBuilding = argsList[1]
        bContinue = argsList[2]
        bTestVisible = argsList[3]
        bIgnoreCost = argsList[4]
        return False

    def cannotConstruct(self,argsList):
        pCity = argsList[0]
        eBuilding = argsList[1]
        bContinue = argsList[2]
        bTestVisible = argsList[3]
        bIgnoreCost = argsList[4]
        pPlayer = gc.getPlayer(pCity.getOwner())
        iBuildingClass = gc.getBuildingInfo(eBuilding).getBuildingClassType()
        eTeam = gc.getTeam(pPlayer.getTeam())

        if pPlayer.getCivics(gc.getInfoTypeForString('CIVICOPTION_CULTURAL_VALUES')) == gc.getInfoTypeForString('CIVIC_CRUSADE'):
            if eBuilding == gc.getInfoTypeForString('BUILDING_ELDER_COUNCIL'):
                return True
            if eBuilding == gc.getInfoTypeForString('BUILDING_MARKET'):
                return True
            if eBuilding == gc.getInfoTypeForString('BUILDING_MONUMENT'):
                return True
            if eBuilding == gc.getInfoTypeForString('BUILDING_MONEYCHANGER'):
                return True
            if eBuilding == gc.getInfoTypeForString('BUILDING_THEATRE'):
                return True
            if eBuilding == gc.getInfoTypeForString('BUILDING_AQUEDUCT'):
                return True
            if eBuilding == gc.getInfoTypeForString('BUILDING_PUBLIC_BATHS'):
                return True
            if eBuilding == gc.getInfoTypeForString('BUILDING_HERBALIST'):
                return True
            if eBuilding == gc.getInfoTypeForString('BUILDING_CARNIVAL'):
                return True
            if eBuilding == gc.getInfoTypeForString('BUILDING_COURTHOUSE'):
                return True
            if eBuilding == gc.getInfoTypeForString('BUILDING_GAMBLING_HOUSE'):
                return True
            if eBuilding == gc.getInfoTypeForString('BUILDING_GRANARY'):
                return True
            if eBuilding == gc.getInfoTypeForString('BUILDING_SMOKEHOUSE'):
                return True
            if eBuilding == gc.getInfoTypeForString('BUILDING_LIBRARY'):
                return True
            if eBuilding == gc.getInfoTypeForString('BUILDING_HARBOR'):
                return True
            if eBuilding == gc.getInfoTypeForString('BUILDING_ALCHEMY_LAB'):
                return True
            if eBuilding == gc.getInfoTypeForString('BUILDING_BREWERY'):
                return True

        if pPlayer.isHuman() == False :
            if eBuilding == gc.getInfoTypeForString('BUILDING_MONUMENT'):
                if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_CREATIVE')) and pCity.getPopulation <=5:
                    return True
            elif eBuilding == gc.getInfoTypeForString('BUILDING_MONEYCHANGER'):
                if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_FINANCIAL')) and pCity.getPopulation <=5:
                    return True
                if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_FINANCIAL')) == False: 
                    if pCity.getPopulation() <=8:
                        return True
            elif eBuilding == gc.getInfoTypeForString('BUILDING_TAX_OFFICE'):
                if pCity.getPopulation() <=12:
                    if pCity.angryPopulation >=0:
                        return True         
            elif eBuilding == gc.getInfoTypeForString('BUILDING_THEATRE'):
                if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_CREATIVE')) and pCity.getPopulation <=6:
                    return True
                if pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_CREATIVE')) == False: 
                    if pCity.getPopulation() <=12:
                        return True
            elif eBuilding == gc.getInfoTypeForString('BUILDING_AQUEDUCT'):
                if pCity.badHealth(False)-pCity.goodHealth() < -1 :
                    return True
            elif eBuilding == gc.getInfoTypeForString('BUILDING_PUBLIC_BATHS'):
                if pCity.angryPopulation <=-1:
                    return True
            elif eBuilding == gc.getInfoTypeForString('BUILDING_HERBALIST'):
                if pCity.badHealth(False)-pCity.goodHealth() < -1 :
                    return True
            elif eBuilding == gc.getInfoTypeForString('BUILDING_CARNIVAL'):
                if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_CREATIVE')) :
                    if pCity.angryPopulation <=-1:
                        return True
            elif eBuilding == gc.getInfoTypeForString('BUILDING_GRANARY'):
                if pCity.badHealth(False)-pCity.goodHealth() < -1 :
                    return True
            elif eBuilding == gc.getInfoTypeForString('BUILDING_BREEDING_PIT'):
                if pCity.getPopulation() <=8:
                    return True     
            elif eBuilding == gc.getInfoTypeForString('BUILDING_RELIQUARY'):
                if pCity.getPopulation() <=8:
                    return True             
            elif eBuilding == gc.getInfoTypeForString('BUILDING_SMOKEHOUSE'):
                if pCity.badHealth(False)-pCity.goodHealth() < -1 :
                    return True
            elif eBuilding == gc.getInfoTypeForString('BUILDING_INFIRMARY'):
                if pCity.badHealth(False)-pCity.goodHealth() < 1 :
                    return True
            elif eBuilding == gc.getInfoTypeForString('BUILDING_INN'):
                if pCity.getPopulation() <=10:
                    return True
            elif eBuilding == gc.getInfoTypeForString('BUILDING_TAVERN'):
                if pCity.getPopulation() <=15:
                    return True
            elif eBuilding == gc.getInfoTypeForString('BUILDING_ALCHEMY_LAB'):
                if pCity.getPopulation() <=10:
                    return True 
            elif eBuilding == gc.getInfoTypeForString('BUILDING_CITY_OF_A_THOUSAND_SLUMS'):
                if pCity.getPopulation() <=20:
                    return True         

            if isWorldWonderClass(iBuildingClass):
                if pCity.getPopulation() <=7 and pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_BARBARIAN'):
                    return True
            elif isNationalWonderClass(iBuildingClass):
                if pCity.getPopulation() <= 4:
                    return True
                    
        if eBuilding == gc.getInfoTypeForString('BUILDING_SHRINE_OF_THE_CHAMPION'):
            iHero = cf.getHero(pPlayer)
            if iHero == -1:
                return True
            if CyGame().isUnitClassMaxedOut(iHero, 0) == False:
                return True
            if pPlayer.getUnitClassCount(iHero) > 0:
                return True

        if eBuilding == gc.getInfoTypeForString('BUILDING_MERCURIAN_GATE'):
            if gc.getGame().isOption(GameOptionTypes.GAMEOPTION_NO_HYBOREM_OR_BASIUM):
                return True
            if pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_THE_ASHEN_VEIL'):
                return True
            if pCity.isCapital():
                return True
            if pPlayer.isHuman() == False:
                if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_EVIL'):
                    return True

        iAltar1 = gc.getInfoTypeForString('BUILDING_ALTAR_OF_THE_LUONNOTAR')
        iAltar2 = gc.getInfoTypeForString('BUILDING_ALTAR_OF_THE_LUONNOTAR_ANOINTED')
        iAltar3 = gc.getInfoTypeForString('BUILDING_ALTAR_OF_THE_LUONNOTAR_BLESSED')
        iAltar4 = gc.getInfoTypeForString('BUILDING_ALTAR_OF_THE_LUONNOTAR_CONSECRATED')
        iAltar5 = gc.getInfoTypeForString('BUILDING_ALTAR_OF_THE_LUONNOTAR_DIVINE')
        iAltar6 = gc.getInfoTypeForString('BUILDING_ALTAR_OF_THE_LUONNOTAR_EXALTED')
        iAltar7 = gc.getInfoTypeForString('BUILDING_ALTAR_OF_THE_LUONNOTAR_FINAL')
        if (eBuilding == iAltar1 or eBuilding == iAltar2 or eBuilding == iAltar3 or eBuilding == iAltar4 or eBuilding == iAltar5 or eBuilding == iAltar6 or eBuilding == iAltar7):
            if pPlayer.getAlignment() == gc.getInfoTypeForString('ALIGNMENT_EVIL'):
                return True
            if eBuilding == iAltar1:
                if (pPlayer.countNumBuildings(iAltar2) > 0 or pPlayer.countNumBuildings(iAltar3) > 0 or pPlayer.countNumBuildings(iAltar4) > 0 or pPlayer.countNumBuildings(iAltar5) > 0 or pPlayer.countNumBuildings(iAltar6) > 0 or pPlayer.countNumBuildings(iAltar7) > 0):
                    return True
            if eBuilding == iAltar2:
                if (pPlayer.countNumBuildings(iAltar3) > 0 or pPlayer.countNumBuildings(iAltar4) > 0 or pPlayer.countNumBuildings(iAltar5) > 0 or pPlayer.countNumBuildings(iAltar6) > 0 or pPlayer.countNumBuildings(iAltar7) > 0):
                    return True
            if eBuilding == iAltar3:
                if (pPlayer.countNumBuildings(iAltar4) > 0 or pPlayer.countNumBuildings(iAltar5) > 0 or pPlayer.countNumBuildings(iAltar6) > 0 or pPlayer.countNumBuildings(iAltar7) > 0):
                    return True
            if eBuilding == iAltar4:
                if (pPlayer.countNumBuildings(iAltar5) > 0 or pPlayer.countNumBuildings(iAltar6) > 0 or pPlayer.countNumBuildings(iAltar7) > 0):
                    return True
            if eBuilding == iAltar5:
                if (pPlayer.countNumBuildings(iAltar6) > 0 or pPlayer.countNumBuildings(iAltar7) > 0):
                    return True
            if eBuilding == iAltar6:
                if pPlayer.countNumBuildings(iAltar7) > 0:
                    return True

        if pPlayer.isHuman() == False:
            if eBuilding == gc.getInfoTypeForString('BUILDING_PROPHECY_OF_RAGNAROK'):
                if pPlayer.getAlignment() != gc.getInfoTypeForString('ALIGNMENT_EVIL'):
                    return True

        if eBuilding == gc.getInfoTypeForString('BUILDING_SMUGGLERS_PORT'):
            if pPlayer.isSmugglingRing() == False:
                return True

        return False

    def canCreate(self,argsList):
        pCity = argsList[0]
        eProject = argsList[1]
        bContinue = argsList[2]
        bTestVisible = argsList[3]
        return False

    def cannotCreate(self,argsList):
        pCity = argsList[0]
        eProject = argsList[1]
        bContinue = argsList[2]
        bTestVisible = argsList[3]
        pPlayer = gc.getPlayer(pCity.getOwner())

        if eProject == gc.getInfoTypeForString('PROJECT_PURGE_THE_UNFAITHFUL'):
            if pPlayer.getStateReligion() == -1:
                return True

        if eProject == gc.getInfoTypeForString('PROJECT_BIRTHRIGHT_REGAINED'):
            if not pPlayer.isFeatAccomplished(FeatTypes.FEAT_GLOBAL_SPELL):
                return True

        if eProject == gc.getInfoTypeForString('PROJECT_STIR_FROM_SLUMBER'):
            if pPlayer.getPlayersKilled() == 0:
                return True

        if eProject == gc.getInfoTypeForString('PROJECT_GENESIS'):
            if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_ILLIANS'):
                return True

        if pPlayer.isHuman() == False:
            if pCity.getPopulation() <= 7:
                return True

            if eProject == gc.getInfoTypeForString('PROJECT_ELEGY_OF_THE_SHEAIM'):
                if CyGame().getGlobalCounter() >= 100:
                    return True

            elif eProject == gc.getInfoTypeForString('PROJECT_HALLOWING_OF_THE_ELOHIM'):
                if CyGame().getGlobalCounter() < 20:
                    return True

            elif eProject == gc.getInfoTypeForString('PROJECT_BANE_DIVINE'):
                # Make sure we stand to gain more than we lose
                eDisciple = gc.getInfoTypeForString('UNITCOMBAT_DISCIPLE')
                pTeam = gc.getTeam(pPlayer.getTeam())
                iRitualValue = 0

                for eLoopPlayer in range(gc.getMAX_PLAYERS()):
                    pLoopPlayer = gc.getPlayer(eLoopPlayer)
                    if pLoopPlayer.isAlive():
                        iPlayerTotal = 0
                        for pUnit in PyPlayer(eLoopPlayer).getUnitList():
                            if pUnit.getUnitCombatType() == eDisciple:
                                iPlayerTotal += 1

                        if eLoopPlayer == pCity.getOwner():
                            iRitualValue -= iPlayerTotal
                        elif pTeam.AI_getWarPlan(pLoopPlayer.getTeam()) != -1:
                            iRitualValue += iPlayerTotal

                if iRitualValue < 5:
                    return True

            elif eProject == gc.getInfoTypeForString('PROJECT_BLOOD_OF_THE_PHOENIX'):
                if pPlayer.getNumUnits() < 40:
                    return True

            elif eProject == gc.getInfoTypeForString('PROJECT_GLORY_EVERLASTING'):
                # Make sure we stand to gain more than we lose
                eDemon = gc.getInfoTypeForString('PROMOTION_DEMON')
                eUndead = gc.getInfoTypeForString('PROMOTION_UNDEAD')
                pTeam = gc.getTeam(pPlayer.getTeam())
                iRitualValue = 0

                for eLoopPlayer in range(gc.getMAX_PLAYERS()):
                    pLoopPlayer = gc.getPlayer(eLoopPlayer)
                    if pLoopPlayer.isAlive():
                        iPlayerTotal = 0
                        for pUnit in PyPlayer(eLoopPlayer).getUnitList():
                            if pUnit.isHasPromotion(eDemon) or pUnit.isHasPromotion(eUndead):
                                iPlayerTotal += 1

                        if eLoopPlayer == pCity.getOwner():
                            iRitualValue -= iPlayerTotal
                        elif pTeam.AI_getWarPlan(pLoopPlayer.getTeam()) != -1:
                            iRitualValue += iPlayerTotal

                if iRitualValue < 5:
                    return True

            elif eProject == gc.getInfoTypeForString('PROJECT_NATURES_REVOLT'):
                # Make sure we stand to gain more than we lose
                eSubdueAnimal = gc.getInfoTypeForString('PROMOTION_SUBDUE_ANIMAL')
                pTeam = gc.getTeam(pPlayer.getTeam())
                iRitualValue = 0

                for eLoopPlayer in range(gc.getMAX_PLAYERS()):
                    pLoopPlayer = gc.getPlayer(eLoopPlayer)
                    if pLoopPlayer.isAlive():
                        iPlayerTotal = 0
                        for pUnit in PyPlayer(eLoopPlayer).getUnitList():
                            if pUnit.isAnimal() or pUnit.isHasPromotion(eSubdueAnimal):
                                iPlayerTotal += 1

                        if eLoopPlayer == pCity.getOwner():
                            iRitualValue += iPlayerTotal
                        elif pTeam.AI_getWarPlan(pLoopPlayer.getTeam()) != -1:
                            iRitualValue -= iPlayerTotal

                if iRitualValue < 5:
                    return True

            elif eProject == gc.getInfoTypeForString('PROJECT_PACT_OF_THE_NILHORN'):
                if gc.getTeam(pPlayer.getTeam()).getHasMetCivCount(True) == 0:
                    return True

            elif eProject == gc.getInfoTypeForString('PROJECT_PURGE_THE_UNFAITHFUL'):
                # Make sure hated religions outnumber the liked ones
                pLeader = gc.getLeaderHeadInfo(pPlayer.getPersonalityType())
                iRitualValue = 0

                for eReligion in range(gc.getNumReligionInfos()):
                    if pPlayer.getStateReligion() != eReligion:
                        for pyCity in PyPlayer(pCity.getOwner()).getCityList():
                            pLoopCity = pyCity.GetCy()
                            if pLoopCity.isHasReligion(eReligion) and not pLoopCity.isHolyCityByType(eReligion):
                                iRitualValue -= pLeader.getReligionWeightModifier(eReligion)
                                
                if iRitualValue < 100:
                    return True

            elif eProject == gc.getInfoTypeForString('PROJECT_SAMHAIN'):
                iCityTarget = (gc.getWorldInfo(CyMap().getWorldSize()).getTargetNumCities() + 1) / 2

                if pPlayer.getNumCities() < iCityTarget or pPlayer.getNumUnits() < 10:
                    return True

            elif eProject == gc.getInfoTypeForString('PROJECT_THE_DRAW'):
                # Must have tech to begin building Ascension immediately
                eOmniscience = gc.getInfoTypeForString('TECH_OMNISCIENCE')
                pTeam = gc.getTeam(pPlayer.getTeam())

                if not pTeam.isHasTech(eOmniscience):
                    # Are we currently researching it?
                    if pPlayer.getCurrentResearch() == eOmniscience():
                        if pPlayer.getResearchTurnsLeft(eOmniscience) > pCity.getProjectProductionTurnsLeft(eProject):
                            return True
                    else:
                        return True

        return False

    def canMaintain(self,argsList):
        pCity = argsList[0]
        eProcess = argsList[1]
        bContinue = argsList[2]
        return False

    def cannotMaintain(self,argsList):
        pCity = argsList[0]
        eProcess = argsList[1]
        bContinue = argsList[2]
        return False

    def AI_chooseTech(self,argsList):
        ePlayer = argsList[0]
        bFree = argsList[1]
        pPlayer = gc.getPlayer(ePlayer)
        pCity = pPlayer.getCapitalCity()
        eTeam = gc.getTeam(pPlayer.getTeam())
        iTech = -1

        # Fireballs for civs without siege (i.e. elves)
        if pPlayer.getCivilizationType() in (gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR'), gc.getInfoTypeForString('CIVILIZATION_SVARTALFAR')):
            if pPlayer.isHasTech(gc.getInfoTypeForString('TECH_KNOWLEDGE_OF_THE_ETHER')) and not pPlayer.isHasTech(gc.getInfoTypeForString('TECH_ELEMENTALISM')):
                if CyGame().getSorenRandNum(100, 'Elven Beeline') <= 50:
                    iTech = gc.getInfoTypeForString('TECH_ELEMENTALISM')
            elif pPlayer.isHasTech(gc.getInfoTypeForString('TECH_ELEMENTALISM')) and not pPlayer.isHasTech(gc.getInfoTypeForString('TECH_SORCERY')):
                iTech = gc.getInfoTypeForString('TECH_SORCERY')

        # Magnadine, etc. (play to Hippus' military strengths)
        if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_HIPPUS'):
            if eTeam.isHasTech(gc.getInfoTypeForString('TECH_IRON_WORKING')):
                if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_WARHORSES')):
                    iTech = gc.getInfoTypeForString('TECH_WARHORSES')

        # Luchuirp fireball/golem beelines
        if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LUCHUIRP'):
            if eTeam.isHasTech(gc.getInfoTypeForString('TECH_WRITING')) :
                if CyGame().getSorenRandNum(100, 'Luchuirp Beeline') <= 50 and eTeam.isHasTech(gc.getInfoTypeForString('TECH_ENGINEERING')) == False:                
                    iTech = gc.getInfoTypeForString('TECH_ENGINEERING')
                elif eTeam.isHasTech(gc.getInfoTypeForString('TECH_ELEMENTALISM')) == False:    
                    iTech = gc.getInfoTypeForString('TECH_ELEMENTALISM')    
                elif eTeam.isHasTech(gc.getInfoTypeForString('TECH_SORCERY')) == False:
                    iTech = gc.getInfoTypeForString('TECH_SORCERY')
                elif eTeam.isHasTech(gc.getInfoTypeForString('TECH_MACHINERY')) == False:
                    if eTeam.isHasTech(gc.getInfoTypeForString('TECH_BOWYERS')):
                        iTech = gc.getInfoTypeForString('TECH_MACHINERY')
                        
        # Advanced religious beelines
        if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CORRUPTION_OF_SPIRIT')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_MALEVOLENT_DESIGNS')) == False:
            if pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_THE_ASHEN_VEIL'):
                if pPlayer.hasHolyCity(pPlayer.getStateReligion()) == True :
                    if eTeam.isHasTech(gc.getInfoTypeForString('TECH_FANATICISM')):
                        iTech = gc.getInfoTypeForString('TECH_MALEVOLENT_DESIGNS')
                        
        if eTeam.isHasTech(gc.getInfoTypeForString('TECH_ORDERS_FROM_HEAVEN')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')) :
            if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_RIGHTEOUSNESS')) :
                if pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_THE_ORDER'):
                    if pPlayer.hasHolyCity(pPlayer.getStateReligion()) == True :
                        if eTeam.isHasTech(gc.getInfoTypeForString('TECH_FANATICISM')):
                            iTech = gc.getInfoTypeForString('TECH_RIGHTEOUSNESS')
        
        if eTeam.isHasTech(gc.getInfoTypeForString('TECH_MESSAGE_FROM_THE_DEEP')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')) :
            if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_ARCANE_LORE')) :
                if pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_OCTOPUS_OVERLORD'):
                    if pPlayer.hasHolyCity(pPlayer.getStateReligion()) == True :
                        if eTeam.isHasTech(gc.getInfoTypeForString('TECH_KNOWLEDGE OF THE ETHER')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_TRADE')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_SANITATION')):
                            iTech = gc.getInfoTypeForString('TECH_ARCANE_LORE')

        if eTeam.isHasTech(gc.getInfoTypeForString('TECH_HONOR')) :
            if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_RELIGIOUS_LAW')) :
                if pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_THE_EMPYREAN'):
                    if pPlayer.hasHolyCity(pPlayer.getStateReligion()) == True :
                        if eTeam.isHasTech(gc.getInfoTypeForString('TECH_PRIESTHOOD')):
                            iTech = gc.getInfoTypeForString('TECH_RELIGIOUS_LAW')   
        
        if eTeam.isHasTech(gc.getInfoTypeForString('TECH_WAY_OF_THE_FORESTS')) :
            if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_COMMUNE_WITH_NATURE')) :
                if pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_FELLOWSHIP_OF_LEAVES'):
                    if pPlayer.hasHolyCity(pPlayer.getStateReligion()) == True :
                        if eTeam.isHasTech(gc.getInfoTypeForString('TECH_PRIESTHOOD')):
                            iTech = gc.getInfoTypeForString('TECH_COMMUNE_WITH_NATURE') 

        # Economic safeguards
        if pPlayer.getNumCities() >= 8 and eTeam.isHasTech(gc.getInfoTypeForString('TECH_CURRENCY')) == False and eTeam.isHasTech(gc.getInfoTypeForString('TECH_CODE_OF_LAWS')):
            iTech = gc.getInfoTypeForString('TECH_CURRENCY') 
        
        if pPlayer.getNumCities() >= 8 and eTeam.isHasTech(gc.getInfoTypeForString('TECH_TRADE')) == False and eTeam.isHasTech(gc.getInfoTypeForString('TECH_WRITING')):
            iTech = gc.getInfoTypeForString('TECH_TRADE') 
        
        if pPlayer.getNumCities() >= 7 and eTeam.isHasTech(gc.getInfoTypeForString('TECH_CONSTRUCTION')) == False :
            iTech = gc.getInfoTypeForString('TECH_CONSTRUCTION')
            
        if pPlayer.getNumCities() >= 6 and eTeam.isHasTech(gc.getInfoTypeForString('TECH_SANITATION')) == False and eTeam.isHasTech(gc.getInfoTypeForString('TECH_BRONZE_WORKING')):
            if not pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_FALLOW')):
                iTech = gc.getInfoTypeForString('TECH_SANITATION') 

        # Normal religious beelines
        if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CORRUPTION_OF_SPIRIT')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_INFERNAL_PACT')) == False:
            if pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_THE_ASHEN_VEIL'):
                if pPlayer.hasHolyCity(pPlayer.getStateReligion()) == True :
                    iTech = gc.getInfoTypeForString('TECH_INFERNAL_PACT')
        
        if eTeam.isHasTech(gc.getInfoTypeForString('TECH_WAY_OF_THE_EARTHMOTHER')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_ARETE')) == False:
            if pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_RUNES_OF_KILMORPH'):
                if pPlayer.hasHolyCity(pPlayer.getStateReligion()) == True :
                    iTech = gc.getInfoTypeForString('TECH_ARETE')
        
        if eTeam.isHasTech(gc.getInfoTypeForString('TECH_MESSAGE_FROM_THE_DEEP')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')) and eTeam.isHasTech(gc.getInfoTypeForString('TECH_MIND_STAPLING')) == False:
            if pPlayer.getStateReligion() == gc.getInfoTypeForString('RELIGION_OCTOPUS_OVERLORDS'):
                if pPlayer.hasHolyCity(pPlayer.getStateReligion()) == True :
                    iTech = gc.getInfoTypeForString('TECH_MIND_STAPLING')

        # Minimum requirements for defence
        if not pPlayer.AI_isDoPeace() or eTeam.getAnyWarPlanCount(True) > 0:
            if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LUCHUIRP'):
                if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_CONSTRUCTION')):
                    iTech = gc.getInfoTypeForString('TECH_CONSTRUCTION')
            elif pPlayer.getCivilizationType() in (gc.getInfoTypeForString('CIVILIZATION_HIPPUS'), gc.getInfoTypeForString('CIVILIZATION_KURIOTATES')):
                if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_HORSEBACK_RIDING')):
                    iTech = gc.getInfoTypeForString('TECH_HORSEBACK_RIDING')
            elif pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR'):
                if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_ARCHERY')):
                    iTech = gc.getInfoTypeForString('TECH_ARCHERY')
            elif pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_SVARTALFAR'):
                if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_HUNTING')):
                    iTech = gc.getInfoTypeForString('TECH_HUNTING')
            elif not eTeam.isHasTech(gc.getInfoTypeForString('TECH_BRONZE_WORKING')):
                iTech = gc.getInfoTypeForString('TECH_BRONZE_WORKING')

        if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR'):
            if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_WAY_OF_THE_FORESTS')) and eTeam.getAnyWarPlanCount(True) == 0:
                iTech = gc.getInfoTypeForString('TECH_WAY_OF_THE_FORESTS')

        # Kurios should get God King ASAP; Ljosalfar usually want Mysticism before Hunting
        if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_MYSTICISM')) and eTeam.getAnyWarPlanCount(True) == 0:
            if pPlayer.getCivilizationType() in (gc.getInfoTypeForString('CIVILIZATION_KURIOTATES'), gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR')):
                iTech = gc.getInfoTypeForString('TECH_MYSTICISM')

        if pPlayer.getNumCities() >= 4 and eTeam.isHasTech(gc.getInfoTypeForString('TECH_EDUCATION')) == False:
            iTech = gc.getInfoTypeForString('TECH_EDUCATION')

        # Khazad need to get a better economy going quickly
        if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_KHAZAD'):
            if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_WAY_OF_THE_EARTHMOTHER')):
                iTech = gc.getInfoTypeForString('TECH_WAY_OF_THE_EARTHMOTHER')
            elif not eTeam.isHasTech(gc.getInfoTypeForString('TECH_FESTIVALS')):
                iTech = gc.getInfoTypeForString('TECH_FESTIVALS')
            elif pPlayer.hasTrait(gc.getInfoTypeForString('TRAIT_FINANCIAL')):
                if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_CODE_OF_LAWS')):
                    iTech = gc.getInfoTypeForString('TECH_CODE_OF_LAWS')
            elif not eTeam.isHasTech(gc.getInfoTypeForString('TECH_CARTOGRAPHY')):
                iTech = gc.getInfoTypeForString('TECH_CARTOGRAPHY')

        # If stranded, prioritise sailing
        if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_SAILING')):
            if pPlayer.countNumCoastalCities() > 0 and pPlayer.AI_getNumAreaCitySites(pCity.area()) == 0:
                iTech = gc.getInfoTypeForString('TECH_SAILING')
            
        # Basic economy
        if eTeam.isHasTech(gc.getInfoTypeForString('TECH_CALENDAR')) == False:
            iTech = gc.getInfoTypeForString('TECH_CALENDAR')
        
        if eTeam.isHasTech(gc.getInfoTypeForString('TECH_MINING')) == False and not gc.getCivilizationInfo(pPlayer.getCivilizationType()).isMaintainFeatures(gc.getInfoTypeForString('FEATURE_FOREST')): 
            iTech = gc.getInfoTypeForString('TECH_MINING')
        
        if eTeam.isHasTech(gc.getInfoTypeForString('TECH_EXPLORATION')) == False:
            iTech = gc.getInfoTypeForString('TECH_EXPLORATION')
                                    
        if pPlayer.getNumCities() >= 4 and eTeam.isHasTech(gc.getInfoTypeForString('TECH_CALENDAR')) == True and eTeam.isHasTech(gc.getInfoTypeForString('TECH_FESTIVALS'))== False:
            if pPlayer.getCivilizationType() != gc.getInfoTypeForString('CIVILIZATION_KHAZAD'):
                iTech = gc.getInfoTypeForString('TECH_FESTIVALS')
        
        if pPlayer.getNumCities() >= 6 and eTeam.isHasTech(gc.getInfoTypeForString('TECH_CARTOGRAPHY')) == False:
            if pPlayer.getCivilizationType() not in (gc.getInfoTypeForString('CIVILIZATION_KURIOTATES'), gc.getInfoTypeForString('CIVILIZATION_KHAZAD')):
                iTech = gc.getInfoTypeForString('TECH_CARTOGRAPHY')
        
        if eTeam.isHasTech(gc.getInfoTypeForString('TECH_MINING')) == False and eTeam.isHasTech(gc.getInfoTypeForString('TECH_CALENDAR')) == False :
            iMining = 0
            iCalendar = 0
            iHunting = 0

            # Deliberately ignore third ring of Kuriotate cities and the city square itself
            for iI in range(1, 21):
                pLoopPlot = pCity.getCityIndexPlot(iI)
                if pLoopPlot:
                    eBonus = pLoopPlot.getBonusType(pPlayer.getTeam())
                    if eBonus in (gc.getInfoTypeForString('BONUS_GOLD'), gc.getInfoTypeForString('BONUS_GEMS')):
                        iMining += 1
                    elif eBonus in (gc.getInfoTypeForString('BONUS_DYE'), gc.getInfoTypeForString('BONUS_INCENSE')):
                        iCalendar += 1
                    elif eBonus == gc.getInfoTypeForString('BONUS_FUR'):
                        iHunting += 1

            if iMining > 0 and iMining >= iCalendar:
                iTech = gc.getInfoTypeForString('TECH_MINING')
            elif iCalendar > 0:
                iTech = gc.getInfoTypeForString('TECH_CALENDAR')
            elif iHunting >= 3:
                iTech = gc.getInfoTypeForString('TECH_HUNTING')
    
        if eTeam.isHasTech(gc.getInfoTypeForString('TECH_AGRICULTURE')) == False:
            iGrain = 0

            # Deliberately ignore third ring of Kuriotate cities and the city square itself
            for iI in range(1, 21):
                pLoopPlot = pCity.getCityIndexPlot(iI)
                if pLoopPlot:
                    eBonus = pLoopPlot.getBonusType(pPlayer.getTeam())

                    if eBonus in (gc.getInfoTypeForString('BONUS_CORN'), gc.getInfoTypeForString('BONUS_WHEAT')):
                        iGrain += 3
                    elif eBonus == gc.getInfoTypeForString('BONUS_RICE'):
                        iGrain += 2

                    if pLoopPlot.isFreshWater():
                        if pLoopPlot.getImprovementType() == -1 and pLoopPlot.getYield(YieldTypes.YIELD_FOOD) >= 2:
                            if pLoopPlot.getFeatureType() != gc.getInfoTypeForString('FEATURE_FOREST') or gc.getCivilizationInfo(pPlayer.getCivilizationType()).isMaintainFeatures(gc.getInfoTypeForString('FEATURE_FOREST')):
                                iGrain += 1

            if iGrain >= 2:
                iTech = gc.getInfoTypeForString('TECH_AGRICULTURE')
    
        if not eTeam.isHasTech(gc.getInfoTypeForString('TECH_CRAFTING')) or not eTeam.isHasTech(gc.getInfoTypeForString('TECH_FISHING')):
            iCrafting = 0
            iSeafood = 0

            # Deliberately ignore third ring of Kuriotate cities and the city square itself
            for iI in range(1, 21):
                pLoopPlot = pCity.getCityIndexPlot(iI)
                if pLoopPlot:
                    eBonus = pLoopPlot.getBonusType(pPlayer.getTeam())
                    if eBonus == gc.getInfoTypeForString('BONUS_WINE'):
                        iCrafting += 1
                    elif eBonus == gc.getInfoTypeForString('BONUS_FISH'):
                        iSeafood += 2
                    elif eBonus in (gc.getInfoTypeForString('BONUS_CLAM'), gc.getInfoTypeForString('BONUS_CRAB'), gc.getInfoTypeForString('BONUS_PEARL')):
                        iSeafood += 1

            if iCrafting >= 2 and not eTeam.isHasTech(gc.getInfoTypeForString('TECH_CRAFTING')):
                iTech = gc.getInfoTypeForString('TECH_CRAFTING')

            elif pPlayer.countNumCoastalCities() > 0 and not eTeam.isHasTech(gc.getInfoTypeForString('TECH_FISHING')):
                if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LANUN'):
                    iSeafood += 2

                if iSeafood >= 2:
                    iTech = gc.getInfoTypeForString('TECH_FISHING')
                        
        if iTech != -1:         
            if eTeam.isHasTech(iTech) == False:
                return iTech

        return TechTypes.NO_TECH

    def AI_chooseProduction(self,argsList):
        pCity = argsList[0]
        pPlayer = gc.getPlayer(pCity.getOwner())
        pPlot = pCity.plot()

        if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_CLAN_OF_EMBERS'):
            if pPlayer.isHasTech(gc.getInfoTypeForString('TECH_MASONRY')):
                if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_WARRENS')) == 0:
                    if pPlot.getNumUnits() >= 2 and pCity.getBaseYieldRate(YieldTypes.YIELD_PRODUCTION) >= 7:
                        pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, gc.getInfoTypeForString('BUILDING_WARRENS'), -1, False, False, False, True)
                        return True

        if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_LUCHUIRP'):
            if pPlayer.isHasTech(gc.getInfoTypeForString('TECH_SORCERY')):
                if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_SCULPTORS_STUDIO')) > 0:
                    if pCity.getNumBuilding(gc.getInfoTypeForString('BUILDING_BLASTING_WORKSHOP')) == 0:
                        if pPlot.getNumUnits() >= 2 and pCity.getBaseYieldRate(YieldTypes.YIELD_PRODUCTION) >= 7:
                            pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, gc.getInfoTypeForString('BUILDING_BLASTING_WORKSHOP'), -1, False, False, False, True)
                            return True

        if pPlayer.isHasTech(gc.getInfoTypeForString('TECH_SANITATION')) and not pPlayer.isBarbarian():
            if pPlayer.getNumBuilding(gc.getInfoTypeForString('BUILDING_PUBLIC_BATHS')) == 0:
                if pCity.angryPopulation(1) > 0:
                    if pPlot.getNumUnits() >= 2 and pCity.getBaseYieldRate(YieldTypes.YIELD_PRODUCTION) >= 5:
                        pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, gc.getInfoTypeForString('BUILDING_PUBLIC_BATHS'), -1, False, False, False, True)
                        return True

        # Siege
        if pPlayer.getCivilizationType() not in (gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR'), gc.getInfoTypeForString('CIVILIZATION_SVARTALFAR')):
            if pPlayer.isHasTech(gc.getInfoTypeForString('TECH_CONSTRUCTION')) and not pPlayer.isBarbarian():
                if pPlayer.getNumBuilding(gc.getInfoTypeForString('BUILDING_SIEGE_WORKSHOP')) == 0:
                    if (pPlayer.getBuildingClassCountPlusMaking(gc.getInfoTypeForString('BUILDINGCLASS_SIEGE_WORKSHOP')) * 3) < pPlayer.getNumCities():
                        if pPlot.getNumUnits() >= 2 and pCity.getBaseYieldRate(YieldTypes.YIELD_PRODUCTION) >= 7:
                            pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, gc.getInfoTypeForString('BUILDING_SIEGE_WORKSHOP'), -1, False, False, False, True)
                            return True

        # The siege alternative
        if pPlayer.getCivilizationType() in (gc.getInfoTypeForString('CIVILIZATION_LJOSALFAR'), gc.getInfoTypeForString('CIVILIZATION_SVARTALFAR')):
            if pPlayer.isHasTech(gc.getInfoTypeForString('TECH_SORCERY')) and pPlayer.isHasTech(gc.getInfoTypeForString('TECH_ELEMENTALISM')):
                if pPlayer.getNumBuilding(gc.getInfoTypeForString('BUILDING_MAGE_GUILD')) == 0:
                    if (pPlayer.getBuildingClassCountPlusMaking(gc.getInfoTypeForString('BUILDINGCLASS_MAGE_GUILD')) * 3) < pPlayer.getNumCities():
                        if pPlot.getNumUnits() >= 2 and pCity.getBaseYieldRate(YieldTypes.YIELD_PRODUCTION) >= 7:
                            pCity.pushOrder(OrderTypes.ORDER_CONSTRUCT, gc.getInfoTypeForString('BUILDING_MAGE_GUILD'), -1, False, False, False, True)
                            return True

        return False

    def AI_unitUpdate(self,argsList):
        pUnit = argsList[0]
        pPlot = pUnit.plot()
        
        if pUnit.getUnitClassType() == gc.getInfoTypeForString('UNITCLASS_GIANT_SPIDER'):
            iX = pUnit.getX()
            iY = pUnit.getY()
            for iiX in range(iX-1, iX+2, 1):
                for iiY in range(iY-1, iY+2, 1):
                    pLoopPlot = CyMap().plot(iiX,iiY)
                    for i in range(pLoopPlot.getNumUnits()):
                        if pLoopPlot.getUnit(i).getOwner() != pUnit.getOwner():
                            return 0
            return 1

        if pUnit.getUnitType() == gc.getInfoTypeForString('UNIT_ACHERON'):
            if pPlot.isVisibleEnemyUnit(pUnit.getOwner()):
                pUnit.cast(gc.getInfoTypeForString('SPELL_BREATH_FIRE'))

        iImprovement = pPlot.getImprovementType()
        if iImprovement != -1:
            if (iImprovement == gc.getInfoTypeForString('IMPROVEMENT_BARROW') or iImprovement == gc.getInfoTypeForString('IMPROVEMENT_RUINS') or iImprovement == gc.getInfoTypeForString('IMPROVEMENT_HELLFIRE')):
                if not pUnit.isAnimal():
                    if pPlot.getNumUnits() - pPlot.getNumAnimalUnits() == 1:
                        return 1
            if (iImprovement == gc.getInfoTypeForString('IMPROVEMENT_BEAR_DEN') or iImprovement == gc.getInfoTypeForString('IMPROVEMENT_LION_DEN')):
                if pUnit.isAnimal():
                    if pPlot.getNumAnimalUnits() == 1:
                        return 1
            if iImprovement == gc.getInfoTypeForString('IMPROVEMENT_GOBLIN_FORT'):
                if not pUnit.isAnimal():
                    if pPlot.getNumUnits() - pPlot.getNumAnimalUnits() <= 3:
                        return 1

        return False

    def AI_doWar(self,argsList):
        eTeam = argsList[0]
        return False

    def AI_doDiplo(self,argsList):
        ePlayer = argsList[0]
        return False

    def AI_goldTarget(self,argsList):
        ePlayer = argsList[0]
        pPlayer = gc.getPlayer(ePlayer)

        # Dwarven vault minimum gold
        if pPlayer.getCivilizationType() == gc.getInfoTypeForString('CIVILIZATION_KHAZAD'):
            iNumCities = pPlayer.getNumCities()
            if iNumCities > 1 and pPlayer.isHasTech(gc.getInfoTypeForString('TECH_FESTIVALS')):
                if pPlayer.isHasTech(gc.getInfoTypeForString('TECH_WAY_OF_THE_EARTHMOTHER')) or pPlayer.isCivic(gc.getInfoTypeForString('CIVIC_ARISTOCRACY')) or pPlayer.isCivic(gc.getInfoTypeForString('CIVIC_CITY_STATES')):
                    if pPlayer.isHasTech(gc.getInfoTypeForString('TECH_TAXATION')) and pPlayer.isHasTech(gc.getInfoTypeForString('TECH_SANITATION')):
                        return iNumCities * 300
                    else:
                        return iNumCities * 150
            else:
                # Grow to normal 'settler' size
                pCity = pPlayer.getCapitalCity()
                if pCity.getPopulation() < 6 and pCity.angryPopulation(1) > 0 and pCity.foodDifference(False) > 1:
                    iGold = pPlayer.getGold()
                    if iGold < 50:
                        return 50
                    if iGold < 100:
                        return 100
                    if iGold < 150:
                        return 150

        return 0

    def AI_isPlotSpecialImprovement(self, argsList):
        pUnit = argsList[0]
        pPlayer = gc.getPlayer(pUnit.getOwner())
        pPlot = argsList[1]
        eImprovement = argsList[2]

        if eImprovement == gc.getInfoTypeForString('IMPROVEMENT_CAGE'):
            for iI in range(pPlot.getNumUnits()):
                pLoopUnit = pPlot.getUnit(iI)
                if pLoopUnit.getOwner() == pUnit.getOwner() and pLoopUnit.isHasPromotion(gc.getInfoTypeForString('PROMOTION_HELD')):
                    return True

        elif eImprovement == gc.getInfoTypeForString('IMPROVEMENT_LETUM_FRIGUS'):
            if pPlot.isPythonActive():
                # Civs other than the Amurites/Illians will trigger this to deny it to them
                return True

        elif eImprovement == gc.getInfoTypeForString('IMPROVEMENT_SEVEN_PINES'):
            if pPlot.isPythonActive():
                if cf.AI_isLosingWar(pPlayer.getTeam()):
                    return True

        elif eImprovement == gc.getInfoTypeForString('IMPROVEMENT_RING_OF_CARCER'):
            # Will never happen, but still...
            if pUnit.getLevel() >= 15:
                return True

        elif eImprovement == gc.getInfoTypeForString('IMPROVEMENT_GRAVEYARD'):
            if pPlayer.getAlignment() != gc.getInfoTypeForString('ALIGNMENT_GOOD'):
                if cf.AI_isLairSafe(pPlayer, pPlot, 10) or not gc.getTeam(pPlayer.getTeam()).isAtWar(gc.getBARBARIAN_TEAM()):
                    return True

        if gc.getTeam(pPlayer.getTeam()).isAtWar(gc.getBARBARIAN_TEAM()):
            if eImprovement in (gc.getInfoTypeForString('IMPROVEMENT_BARROW'),
                                gc.getInfoTypeForString('IMPROVEMENT_DUNGEON'),
                                gc.getInfoTypeForString('IMPROVEMENT_GOBLIN_FORT'),
                                gc.getInfoTypeForString('IMPROVEMENT_RUINS'),
                                gc.getInfoTypeForString('IMPROVEMENT_SHIP_WRECK')):
                if cf.AI_isLairSafe(pPlayer, pPlot, 10):
                    return True

            elif eImprovement in (gc.getInfoTypeForString('IMPROVEMENT_AIFON_ISLE'),
                                  gc.getInfoTypeForString('IMPROVEMENT_BRADELINES_WELL'),
                                  gc.getInfoTypeForString('IMPROVEMENT_BROKEN_SEPULCHER'),
                                  gc.getInfoTypeForString('IMPROVEMENT_PYRE_OF_THE_SERAPHIC')):
                if cf.AI_isLairSafe(pPlayer, pPlot, 20):
                    return True

        return False

    def calculateScore(self,argsList):
        ePlayer = argsList[0]
        bFinal = argsList[1]
        bVictory = argsList[2]
        
        iPopulationScore = CvUtil.getScoreComponent(gc.getPlayer(ePlayer).getPopScore(), gc.getGame().getInitPopulation(), gc.getGame().getMaxPopulation(), gc.getDefineINT("SCORE_POPULATION_FACTOR"), True, bFinal, bVictory)
        iLandScore = CvUtil.getScoreComponent(gc.getPlayer(ePlayer).getLandScore(), gc.getGame().getInitLand(), gc.getGame().getMaxLand(), gc.getDefineINT("SCORE_LAND_FACTOR"), True, bFinal, bVictory)
        iTechScore = CvUtil.getScoreComponent(gc.getPlayer(ePlayer).getTechScore(), gc.getGame().getInitTech(), gc.getGame().getMaxTech(), gc.getDefineINT("SCORE_TECH_FACTOR"), True, bFinal, bVictory)
        iWondersScore = CvUtil.getScoreComponent(gc.getPlayer(ePlayer).getWondersScore(), gc.getGame().getInitWonders(), gc.getGame().getMaxWonders(), gc.getDefineINT("SCORE_WONDER_FACTOR"), False, bFinal, bVictory)
        return int(iPopulationScore + iLandScore + iWondersScore + iTechScore)

    def doHolyCity(self):
        return False

    def doHolyCityTech(self,argsList):
        eTeam = argsList[0]
        ePlayer = argsList[1]
        eTech = argsList[2]
        bFirst = argsList[3]
        return False

    def doGold(self,argsList):
        ePlayer = argsList[0]
        return False

    def doResearch(self,argsList):
        ePlayer = argsList[0]
        return False

    def doGoody(self,argsList):
        ePlayer = argsList[0]
        pPlot = argsList[1]
        pUnit = argsList[2]
        return False

    def doGrowth(self,argsList):
        pCity = argsList[0]
        return False

    def doProduction(self,argsList):
        pCity = argsList[0]
        return False

    def doCulture(self,argsList):
        pCity = argsList[0]
        return False

    def doPlotCulture(self,argsList):
        pCity = argsList[0]
        bUpdate = argsList[1]
        ePlayer = argsList[2]
        iCultureRate = argsList[3]
        return False

    def doReligion(self,argsList):
        pCity = argsList[0]
        return False

    def cannotSpreadReligion(self,argsList):
        iOwner, iUnitID, iReligion, iX, iY = argsList[0]
        return False

    def doGreatPeople(self,argsList):
        pCity = argsList[0]
        return False

    def doMeltdown(self,argsList):
        pCity = argsList[0]
        return False
    
    def doReviveActivePlayer(self,argsList):
        "allows you to perform an action after an AIAutoPlay"
        iPlayer = argsList[0]
        return False
    
    def doPillageGold(self, argsList):
        "controls the gold result of pillaging"
        pPlot = argsList[0]
        pUnit = argsList[1]
        
        iPillageGold = 0
        iPillageGold = CyGame().getSorenRandNum(gc.getImprovementInfo(pPlot.getImprovementType()).getPillageGold(), "Pillage Gold 1")
        iPillageGold += CyGame().getSorenRandNum(gc.getImprovementInfo(pPlot.getImprovementType()).getPillageGold(), "Pillage Gold 2")

        iPillageGold += (pUnit.getPillageChange() * iPillageGold) / 100
        
        return iPillageGold
    
    def doCityCaptureGold(self, argsList):
        "controls the gold result of capturing a city"
        
        pOldCity = argsList[0]
        
        iCaptureGold = 0
        
        iCaptureGold += gc.getDefineINT("BASE_CAPTURE_GOLD")
        iCaptureGold += (pOldCity.getPopulation() * gc.getDefineINT("CAPTURE_GOLD_PER_POPULATION"))
        iCaptureGold += CyGame().getSorenRandNum(gc.getDefineINT("CAPTURE_GOLD_RAND1"), "Capture Gold 1")
        iCaptureGold += CyGame().getSorenRandNum(gc.getDefineINT("CAPTURE_GOLD_RAND2"), "Capture Gold 2")

        if (gc.getDefineINT("CAPTURE_GOLD_MAX_TURNS") > 0):
            iCaptureGold *= cyIntRange((CyGame().getGameTurn() - pOldCity.getGameTurnAcquired()), 0, gc.getDefineINT("CAPTURE_GOLD_MAX_TURNS"))
            iCaptureGold /= gc.getDefineINT("CAPTURE_GOLD_MAX_TURNS")
        
        return iCaptureGold
    
    def citiesDestroyFeatures(self,argsList):
        iX, iY= argsList
        return True
        
    def canFoundCitiesOnWater(self,argsList):
        iX, iY= argsList
        return False
        
    def doCombat(self,argsList):
        pSelectionGroup, pDestPlot = argsList
        return False

    def getConscriptUnitType(self, argsList):
        iPlayer = argsList[0]
        iConscriptUnitType = -1 #return this with the value of the UNIT TYPE you want to be conscripted, -1 uses default system
        
        return iConscriptUnitType

    def getCityFoundValue(self, argsList):
        iPlayer, iPlotX, iPlotY = argsList
        iFoundValue = -1 # Any value besides -1 will be used
        
        return iFoundValue
        
    def canPickPlot(self, argsList):
        pPlot = argsList[0]
        return true
        
    def getUnitCostMod(self, argsList):
        iPlayer, iUnit = argsList
        iCostMod = -1 # Any value > 0 will be used
        
        return iCostMod

    def getBuildingCostMod(self, argsList):
        iPlayer, iCityID, iBuilding = argsList
        pPlayer = gc.getPlayer(iPlayer)
        pCity = pPlayer.getCity(iCityID)

        iCostMod = -1 # Any value > 0 will be used

        if iBuilding == gc.getInfoTypeForString('BUILDING_GAMBLING_HOUSE'):
            if pPlayer.isGamblingRing():
                iCostMod = gc.getBuildingInfo(iBuilding).getProductionCost() / 4
        
        return iCostMod
        
    def canUpgradeAnywhere(self, argsList):
        pUnit = argsList
        
        bCanUpgradeAnywhere = 0
        
        return bCanUpgradeAnywhere
        
    def getWidgetHelp(self, argsList):
        eWidgetType, iData1, iData2, bOption = argsList
        
        return u""
        
    def getUpgradePriceOverride(self, argsList):
        iPlayer, iUnitID, iUnitTypeUpgrade = argsList
        
        return -1   # Any value 0 or above will be used
    
    def getExperienceNeeded(self, argsList):
        # use this function to set how much experience a unit needs
        iLevel, iOwner = argsList
        
        iExperienceNeeded = 0

        # regular epic game experience      
        iExperienceNeeded = iLevel * iLevel + 1

        iModifier = gc.getPlayer(iOwner).getLevelExperienceModifier()
        if (0 != iModifier):
            iExperienceNeeded += (iExperienceNeeded * iModifier + 99) / 100   # ROUND UP
            
        return iExperienceNeeded

